/***************************************************************************
                          schannellist.cpp  -  description
                             -------------------
    begin                : Mon Jun 30 2003
    copyright            : (C) 2003 by Andre Stein
    email                : stone@steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "schannellist.h"
#include <fstream>
#include <iostream>
#include <string>

using namespace std;


/***************************************************************************
   constructor/destructor
 ***************************************************************************/
 
SChannelList::SChannelList()
{
}

SChannelList::~SChannelList()
{
}

/***************************************************************************
   functions
 ***************************************************************************/
 
/** loads channelist from file */
void SChannelList::loadFrom(const char* filename)
{
	ifstream file(filename);
	string testLine;
	SConfType confType;

	//file couldn't be loaded
	if (!file)
	{
		cerr << "Qdvbzap: error  couldn't load " << filename << "." << endl;
		return;
	}

	/**
	 get type of config file
	 **/

	file >> testLine;
	confType = getTypeOfConfFile(testLine.c_str());
	cout << testLine << "type of channels.conf: " << confType << endl;


        //magic double loop... uhlala
        while ( !file.eof() )
        {
		QString tokens[9];
		SChannel channel;
		int i; char c;

		//example line:
		//     DVB-S: Das Erste:11837:h:0:27500:101:102:1:28106
		//     DVB-C: 3sat:394000000:INVERSION_OFF:6900000:FEC_NONE:QAM_64:210:220
		//     DVB-T: BC-Choice:578000000:INVERSION_OFF:BANDWIDTH_8_MHZ:FEC_2_3:FEC_NONE:QAM_64:TRANSMISSION_MODE_2K:GUARD_INTERVAL_1_32:HIERARCHY_NONE:620:621
		//go through all chars and filter tokens
		//looks ssooo nice man
		for (i = 0, c = file.get();
		     (i < 9) && (!file.eof());
		     c = file.get())
		{
			if ( c == '\n')
			{
				break;
				
			}
			else if ( c != ':' )
			{
				tokens[i] += c;
			}
			else
			{
				++i;
			}
		}

		//at least 4 tokens..
		if (i >= 4)
		{
			//fill struct from tokens we got	
			channel.name = tokens[0];
			channel.frequency = tokens[1].toInt();
			channel.polarisation = (tokens[2].at(0) == 'h' ? POL_H : POL_V);
			channel.symbolRate = tokens[4].toInt();
			channel.pids[0] = tokens[5].toInt();
			channel.pids[1] = tokens[6].toInt();
			channel.pids[2] = tokens[7].toInt();
		//	channel.pids[3] = tokens[8].toInt();
			//add to channels array
			channels.push_back(channel);
        	}
        }
}

/** saves channellist to file */
void SChannelList::saveTo(const char* filename)
{
	ofstream file(filename, ios::out | ios::trunc);

	cout << "saving file " << filename << endl;
	
	//fuck you, don't get a file ;)
	if (!file)
	{
		cerr << "Qdvbzap: error  couldn't save to " << filename << "." << endl;
		return;
	}

	//go through and save each one..
	for (unsigned int i = 0; i < channels.size(); ++i)
	{
		file << channels[i].name.latin1() << ':'
		     << channels[i].frequency << ':'
		     << (channels[i].polarisation == POL_H ? 'h' : 'v') << ':'
		     << "0:"        //don't know this one??
		     << channels[i].symbolRate << ':'
		     << channels[i].pids[0] << ':'
		     << channels[i].pids[1] << ':'
		     << channels[i].pids[2] << ':';
	//	     << channels[i].pids[3] << endl;
	}

	//wasn't really incredible. out for a smoke
}

/** adds a channel to the liszt */
void SChannelList::addChannel(QString name, int frequency, bool polarisation, int symbolRate, int pid1, int pid2, int pid3, int pid4)
{
	//setup channel struture
	SChannel toAdd;
	toAdd.name = name;
	toAdd.frequency = frequency;
	toAdd.polarisation = polarisation;
	toAdd.symbolRate = symbolRate;
	toAdd.pids[0] = pid1;
	toAdd.pids[1] = pid2;
	toAdd.pids[2] = pid3;
	//toAdd.pids[3] = pid4;

	//add to vector of channels
	channels.push_back(toAdd);
}

/** lookups a channel in the list and returns it */
SChannel* SChannelList::lookup(QString channelName)
{
	//go through it and stop when found
	for (unsigned int i = 0; i < channels.size(); ++i)
	{
		//found!
		if (channels[i].name == channelName)
			return &channels[i];
	}

	//nothing
	return (SChannel*)0;
}
/** get next chanel in array until the end */
SChannel* SChannelList::iterateNextChannel()
{
	static int i = -1;

        //next one
        ++i;
	
	//if we have iterated through array,
	//reset and return 0 for done.
	if ( i >= int(channels.size()) )
	{
		i = -1;
		return 0;
	}

	//return current
	return &channels[i];
}

/** remove channel with given name */
void SChannelList::removeChannel(QString name)
{
	for (unsigned int i = 0; i < channels.size(); ++i)
	{
		if (channels[i].name == name)
		{
			channels.erase(channels.begin() + i);
			break;
		}
	}
}

/** returns type of config file using example line **/
SConfType SChannelList::getTypeOfConfFile(const char* aLine)
{
	//counter that counts how many tokens (:) this line contains
	int occurrence = 0;
	int size = strlen(aLine);

	for (int i = 0; i < size; ++i)
	{
		if (aLine[i] == ':')
			occurrence++;
	}

	/** depending of number of tokens, return conf type **/
	if (occurrence == 7)
		return DVB_C;
	else if (occurrence == 8)
		return DVB_S;
	else
		return DVB_T;

	//our job is done.
}
