/***************************************************************************
                          schannellist.h  -  description
                             -------------------
    begin                : Mon Jun 30 2003
    copyright            : (C) 2003 by Andre Stein
    email                : stone@steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCHANNELLIST_H
#define SCHANNELLIST_H

#include <qstring.h>
#include <vector>

using namespace std;

#define POL_H true
#define POL_V false

/**
  * represents one little channel
  */
struct SChannel
{
	SChannel()
	: frequency(0), symbolRate(0), polarisation(POL_H)
	{
		pids[0] = pids[1] = pids[2] = pids[3] = 0;
	}
	
	QString name;
	int frequency;
	int symbolRate;
	/* true: H, false: V */
	bool polarisation;
	int pids[4];
};

//type of DVB used
enum SConfType
{
	DVB_S,
	DVB_C,
	DVB_T
};

/** This class represents the channel list. It manages all
  * available channels, saves them to and loads them from disk.
  * @author Andre Stein
  */
class SChannelList
{
public: 
	SChannelList();
	~SChannelList();

	/** loads channelist from file */
	void loadFrom(const char* filename);
	/** saves channellist to file */
	void saveTo(const char* filename);
	/** adds a channel to the liszt */
	void addChannel(QString name, int frequency, bool polarisation, int symbolRate, int pid1, int pid2, int pid3, int pid4);
	/** lookups a channel in the list and returns it */
	SChannel* lookup(QString channelName);
	/** get next chanel in array until the end*/
	SChannel* iterateNextChannel();
	/** remove channel with given name */
	void removeChannel(QString name);
	/** clear channels */
	void clear() { channels.clear(); }
protected:
	/** returns type of file **/
	SConfType getTypeOfConfFile(const char* aLine);

	vector<SChannel> channels;
};

#endif
